Package: mlS3
Type: Package
Title: Unified S3 Interface to Machine Learning Models
Version: 0.1.1
Date: 2026-05-24
Authors@R: 
    person("T.", "Moudiki",
           email = "thierry.moudiki@gmail.com",
           role = c("aut", "cre"))
Maintainer: T. Moudiki <thierry.moudiki@gmail.com>
Description: Provides a unified and consistent S3 interface for training and predicting with a variety of machine learning models in R. The package wraps popular algorithms (e.g., from 'glmnet', 'lightgbm', 'ranger', 'e1071', and 'caret') under a common workflow based on simple wrap_*() and predict() functions, allowing users to switch between models without changing their code structure. It supports both classification and regression tasks and facilitates rapid experimentation, benchmarking, and comparison of models. By abstracting away package-specific APIs while preserving flexibility in parameter specification, the package streamlines machine learning workflows and promotes reproducibility.
License: GPL-3
Encoding: UTF-8
Imports: e1071, glmnet, lightgbm, ranger
Suggests: caret, knitr, randomForest, kernlab
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Language: en-US
NeedsCompilation: no
Packaged: 2026-05-27 22:20:47 UTC; t
Author: T. Moudiki [aut, cre]
Repository: CRAN
Date/Publication: 2026-05-28 07:30:16 UTC
Built: R 4.6.0; ; 2026-05-28 11:54:41 UTC; unix

export(wrap_glmnet)
export(wrap_lightgbm)
export(wrap_ranger)
export(wrap_svm)
export(wrap_caret)
S3method(predict, wrap_glmnet)
S3method(predict, wrap_lightgbm)
S3method(predict, wrap_ranger)
S3method(predict, wrap_svm)
S3method(predict, wrap_caret)
importFrom("stats", "predict")
importFrom("e1071", "svm")
importFrom("glmnet", "glmnet")
importFrom("lightgbm", "lightgbm")
importFrom("ranger", "ranger")

## ----fig.width=7--------------------------------------------------------------
library(mlS3)

# =============================================================================
# Classification examples (no leakage)
# =============================================================================
set.seed(123)

# --- Binary classification: iris setosa vs versicolor ---
iris_bin <- iris[iris$Species != "virginica", ]
X_bin <- iris_bin[, 1:4]
y_bin <- droplevels(iris_bin$Species)

# Split into train/test
idx_bin <- sample(nrow(X_bin), 0.7 * nrow(X_bin))
X_bin_train <- X_bin[idx_bin, ]
y_bin_train <- y_bin[idx_bin]
X_bin_test  <- X_bin[-idx_bin, ]
y_bin_test  <- y_bin[-idx_bin]

# glmnet
mod <- wrap_glmnet(X_bin_train, y_bin_train, family = "binomial")
pred_bin_glmnet <- predict(mod, newx = X_bin_test, type = "class")
acc_glmnet <- mean(pred_bin_glmnet == y_bin_test)

cat("Accuracy (glmnet): ", acc_glmnet, "\n")


# --- Multiclass classification: iris all species ---
X_multi <- iris[, 1:4]
y_multi <- iris$Species

# Split into train/test
idx_multi <- sample(nrow(X_multi), 0.7 * nrow(X_multi))
X_multi_train <- X_multi[idx_multi, ]
y_multi_train <- y_multi[idx_multi]
X_multi_test  <- X_multi[-idx_multi, ]
y_multi_test  <- y_multi[-idx_multi]

# lightgbm
mod <- wrap_lightgbm(X_multi_train, y_multi_train,
                     params = list(objective = "multiclass",
                                   num_class = 3, verbose = -1),
                     nrounds = 150)
pred_multi_lightgbm <- predict(mod, newx = X_multi_test, type = "class")
acc_lightgbm <- mean(pred_multi_lightgbm == y_multi_test)

# ranger
mod <- wrap_ranger(X_multi_train, y_multi_train, num.trees = 100L)
pred_multi_ranger <- predict(mod, newx = X_multi_test, type = "class")
acc_ranger <- mean(pred_multi_ranger == y_multi_test)

# svm
mod <- wrap_svm(X_multi_train, y_multi_train, kernel = "radial")
pred_multi_svm <- predict(mod, newx = X_multi_test, type = "class")
acc_svm <- mean(pred_multi_svm == y_multi_test)

cat("Accuracy (lightgbm): ", acc_lightgbm, "\n")
cat("Accuracy (ranger): ", acc_ranger, "\n")
cat("Accuracy (svm): ", acc_svm, "\n")

## ----fig.width=7--------------------------------------------------------------
# =============================================================================
# Regression examples (mtcars)
# =============================================================================
X_reg <- mtcars[, -1]
y_reg <- mtcars$mpg

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_reg_train <- X_reg[idx_reg, ];  y_reg_train <- y_reg[idx_reg]
X_reg_test  <- X_reg[-idx_reg, ]; y_reg_test  <- y_reg[-idx_reg]


# lightgbm
mod <- wrap_lightgbm(X_reg_train, y_reg_train,
                     params = list(objective = "regression", verbose = -1),
                     nrounds = 50)
pred_reg_lightgbm <- predict(mod, newx = X_reg_test)
rmse_lightgbm <- sqrt(mean((pred_reg_lightgbm - y_reg_test)^2))

# glmnet
mod <- wrap_glmnet(X_reg_train, y_reg_train, alpha = 0)
pred_reg_glmnet <- predict(mod, newx = X_reg_test)
rmse_glmnet <- sqrt(mean((pred_reg_glmnet - y_reg_test)^2))

# svm
mod <- wrap_svm(X_reg_train, y_reg_train)
pred_reg_svm <- predict(mod, newx = X_reg_test)
rmse_svm <- sqrt(mean((pred_reg_svm - y_reg_test)^2))

# ranger
mod <- wrap_ranger(X_reg_train, y_reg_train, num.trees = 100L)
pred_reg_ranger <- predict(mod, newx = X_reg_test)
rmse_ranger <- sqrt(mean((pred_reg_ranger - y_reg_test)^2))

cat("RMSE (lightgbm): ", rmse_lightgbm, "\n")
cat("RMSE (glmnet): ", rmse_glmnet, "\n")
cat("RMSE (svm): ", rmse_svm, "\n")
cat("RMSE (ranger): ", rmse_ranger, "\n")

## ----regression, fig.width=7--------------------------------------------------
# Load required libraries
library(mlS3)
library(caret)  # Will be suggested, but needed for this example

# ============================================================================
# Regression with mtcars dataset
# ============================================================================
data(mtcars)

# Prepare data
X_reg <- mtcars[, -1]  # All except mpg
y_reg <- mtcars$mpg     # Target variable

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_reg_train <- X_reg[idx_reg, ]
y_reg_train <- y_reg[idx_reg]
X_reg_test <- X_reg[-idx_reg, ]
y_reg_test <- y_reg[-idx_reg]

# ----------------------------------------------------------------------------
# Example 1: Random Forest with specific parameters
# ----------------------------------------------------------------------------
cat("\n=== Example 1: Random Forest Regression ===\n")

mod_rf <- wrap_caret(X_reg_train, y_reg_train, 
                     method = "rf", 
                     mtry = 3)        # Number of variables sampled at each split

print(mod_rf)

# Predictions
pred_rf <- predict(mod_rf, newx = X_reg_test)
rmse_rf <- sqrt(mean((pred_rf - y_reg_test)^2))
r2_rf <- 1 - sum((y_reg_test - pred_rf)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_rf, 3), "\n")
cat("R-squared:", round(r2_rf, 3), "\n")

# ----------------------------------------------------------------------------
# Example 3: Support Vector Machine
# ----------------------------------------------------------------------------
cat("\n=== Example 3: SVM Regression ===\n")

mod_svm <- wrap_caret(X_reg_train, y_reg_train,
                      method = "svmRadial",
                      sigma = 0.1,
                      C = 1)

print(mod_svm)

pred_svm <- predict(mod_svm, newx = X_reg_test)
rmse_svm <- sqrt(mean((pred_svm - y_reg_test)^2))
r2_svm <- 1 - sum((y_reg_test - pred_svm)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_svm, 3), "\n")
cat("R-squared:", round(r2_svm, 3), "\n")

# ----------------------------------------------------------------------------
# Example 4: Neural Network
# ----------------------------------------------------------------------------
cat("\n=== Example 4: Neural Network Regression ===\n")

mod_nnet <- wrap_caret(X_reg_train, y_reg_train,
                       method = "nnet",
                       size = 5,      # Number of hidden units
                       decay = 0.01)  # Weight decay

print(mod_nnet)

pred_nnet <- predict(mod_nnet, newx = X_reg_test)
rmse_nnet <- sqrt(mean((pred_nnet - y_reg_test)^2))
r2_nnet <- 1 - sum((y_reg_test - pred_nnet)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_nnet, 3), "\n")
cat("R-squared:", round(r2_nnet, 3), "\n")

# ----------------------------------------------------------------------------
# Example 5: Compare all models
# ----------------------------------------------------------------------------
cat("\n=== Model Comparison ===\n")
results_reg <- data.frame(
  Model = c("Random Forest", "SVM", "Neural Network"),
  RMSE = round(c(rmse_rf, rmse_svm, rmse_nnet), 3),
  R2 = round(c(r2_rf, r2_svm, r2_nnet), 3)
)
print(results_reg)

# Best model
best_idx <- which.min(results_reg$RMSE)
cat("\nBest model (lowest RMSE):", results_reg$Model[best_idx], "\n")

## ----classification, fig.width=7----------------------------------------------
# ============================================================================
# Classification with iris dataset
# ============================================================================
data(iris)

# ----------------------------------------------------------------------------
# Example 1: Binary Classification (setosa vs versicolor)
# ----------------------------------------------------------------------------
cat("\n=== Example 1: Binary Classification ===\n")

iris_bin <- iris[iris$Species != "virginica", ]
X_bin <- iris_bin[, 1:4]
y_bin <- droplevels(iris_bin$Species)

# Split
set.seed(123)
idx_bin <- sample(nrow(X_bin), 0.7 * nrow(X_bin))
X_bin_train <- X_bin[idx_bin, ]
y_bin_train <- y_bin[idx_bin]
X_bin_test <- X_bin[-idx_bin, ]
y_bin_test <- y_bin[-idx_bin]

# Random Forest
mod_rf_bin <- wrap_caret(X_bin_train, y_bin_train,
                         method = "rf",
                         mtry = 2)

print(mod_rf_bin)

# Class predictions
pred_rf_bin <- predict(mod_rf_bin, newx = X_bin_test, type = "class")
acc_rf <- mean(pred_rf_bin == y_bin_test)

# Probability predictions
prob_rf_bin <- predict(mod_rf_bin, newx = X_bin_test, type = "prob")
head(prob_rf_bin)

cat("Accuracy:", round(acc_rf, 4), "\n")
cat("Confusion Matrix:\n")
print(table(Predicted = pred_rf_bin, Actual = y_bin_test))

# ----------------------------------------------------------------------------
# Example 2: Multiclass Classification (all species)
# ----------------------------------------------------------------------------
cat("\n=== Example 2: Multiclass Classification ===\n")

X_multi <- iris[, 1:4]
y_multi <- iris$Species

# Split
set.seed(123)
idx_multi <- sample(nrow(X_multi), 0.7 * nrow(X_multi))
X_multi_train <- X_multi[idx_multi, ]
y_multi_train <- y_multi[idx_multi]
X_multi_test <- X_multi[-idx_multi, ]
y_multi_test <- y_multi[-idx_multi]

# Random Forest
mod_rf_multi <- wrap_caret(X_multi_train, y_multi_train,
                           method = "rf",
                           mtry = 2)

pred_rf_multi <- predict(mod_rf_multi, newx = X_multi_test, type = "class")
acc_rf_multi <- mean(pred_rf_multi == y_multi_test)
cat("Random Forest Accuracy:", round(acc_rf_multi, 4), "\n")

# ----------------------------------------------------------------------------
# Example 3: Support Vector Machine (multiclass)
# ----------------------------------------------------------------------------
cat("\n=== Example 3: SVM Multiclass ===\n")

mod_svm_multi <- wrap_caret(X_multi_train, y_multi_train,
                            method = "svmRadial",
                            sigma = 0.1,
                            C = 10)

pred_svm_multi <- predict(mod_svm_multi, newx = X_multi_test, type = "class")
acc_svm_multi <- mean(pred_svm_multi == y_multi_test)
cat("SVM Accuracy:", round(acc_svm_multi, 4), "\n")

# Confusion Matrix
cat("\nSVM Confusion Matrix:\n")
print(table(Predicted = pred_svm_multi, Actual = y_multi_test))

# ----------------------------------------------------------------------------
# Example 4: K-Nearest Neighbors
# ----------------------------------------------------------------------------
cat("\n=== Example 4: KNN Classification ===\n")

mod_knn <- wrap_caret(X_multi_train, y_multi_train,
                      method = "knn",
                      k = 5)

pred_knn <- predict(mod_knn, newx = X_multi_test, type = "class")
acc_knn <- mean(pred_knn == y_multi_test)
cat("KNN Accuracy:", round(acc_knn, 4), "\n")

# ----------------------------------------------------------------------------
# Example 5: Linear Discriminant Analysis
# ----------------------------------------------------------------------------
cat("\n=== Example 5: LDA Classification ===\n")

mod_lda <- wrap_caret(X_multi_train, y_multi_train,
                      method = "lda")

pred_lda <- predict(mod_lda, newx = X_multi_test, type = "class")
acc_lda <- mean(pred_lda == y_multi_test)
cat("LDA Accuracy:", round(acc_lda, 4), "\n")

# Get posterior probabilities
prob_lda <- predict(mod_lda, newx = X_multi_test, type = "prob")
cat("\nSample probabilities (first 3 rows):\n")
print(head(prob_lda, 3))

# ----------------------------------------------------------------------------
# Example 7: Compare all classification models
# ----------------------------------------------------------------------------
cat("\n=== Model Comparison (Multiclass) ===\n")
results_class <- data.frame(
  Model = c("Random Forest", "SVM", "KNN", "LDA"),
  Accuracy = round(c(acc_rf_multi, acc_svm_multi, acc_knn, acc_lda), 4)
)
print(results_class[order(-results_class$Accuracy), ])

# ----------------------------------------------------------------------------
# Example 8: Advanced - Probability calibration check
# ----------------------------------------------------------------------------
cat("\n=== Probability Distributions (LDA) ===\n")

# Get probabilities for all classes
probs <- predict(mod_lda, newx = X_multi_test, type = "prob")

# Show distribution of predicted probabilities for each class
oldpar <- par(mfrow = c(1, 3))
on.exit(par(oldpar))
for (class_name in colnames(probs)) {
  hist(probs[, class_name], 
       main = paste("Probabilities for", class_name),
       xlab = "Probability",
       col = "lightblue",
       breaks = 20)
}

#' S3 wrapper for ranger
#'
#' Fits a `ranger` random forest with a consistent interface.
#' Supports both classification (factor `y`) and regression (numeric `y`).
#'
#' @param x A matrix or data.frame of features.
#' @param y A factor or character vector for classification, numeric for regression.
#' @param ... Additional arguments passed to [ranger::ranger()].
#' @return An object of class `wrap_ranger` with fields:
#'   \item{fit}{The fitted ranger model.}
#'   \item{levels}{Class levels (NULL for regression).}
#'   \item{task}{"classification" or "regression".}
#'
#' @examples
#'
#' X <- as.matrix(iris[, 1:4])
#' y <- iris$Species
#' mod <- wrap_ranger(X, y, num.trees = 100L)
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
#' @export
wrap_ranger <- function(x, y, ...) {
  if (!requireNamespace("ranger", quietly = TRUE))
    stop("Package 'ranger' required. Install with: install.packages('ranger')")
  x <- .std_colnames(.coerce_newx(x, "data.frame"))
  if (.is_classif(y)) y <- .to_factor(y)
  df  <- data.frame(y = y, x)
  fit <- ranger::ranger(y ~ ., data = df, probability = .is_classif(y), ...)
  .wrap(fit, y, "wrap_ranger")
}

#' @rdname wrap_ranger
#' @param object A fitted `wrap_ranger` object.
#' @param newx A matrix or data.frame of new observations.
#' @param type `"class"` (default) for class labels, `"prob"` for a probability
#'   matrix. Ignored for regression.
#' @export
#'
#' @examples
#'
#' X <- as.matrix(iris[, 1:4])
#' y <- iris$Species
#' mod <- wrap_ranger(X, y, num.trees = 100L)
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
predict.wrap_ranger <- function(object, newx, type = c("class", "prob"), ...) {
  newx <- .std_colnames(.coerce_newx(newx, "data.frame"))
  raw  <- predict(object$fit, data = newx)$predictions
  if (object$task == "regression") return(as.numeric(raw))
  colnames(raw) <- object$levels
  .format_output(raw, match.arg(type), object$levels, object$task)
}

#' @rdname wrap_ranger
#' @export
print.wrap_ranger <- function(x, ...) {
  cat("wrap_ranger\n")
  cat("  task    :", x$task, "\n")
  if (!is.null(x$levels))
    cat("  classes :", paste(x$levels, collapse = ", "), "\n")
  cat("  trees   :", x$fit$num.trees, "\n")
  cat("  features:", x$fit$num.independent.variables, "\n")
  invisible(x)
}

X_bin   <- as.matrix(iris[iris$Species != "virginica", 1:4])
y_bin   <- droplevels(iris[iris$Species != "virginica", "Species"])
X_multi <- as.matrix(iris[, 1:4])
y_multi <- iris$Species
X_reg   <- as.matrix(mtcars[, -1])
y_reg   <- mtcars$mpg

is_prob_matrix <- function(m, k) {
  is.matrix(m) && ncol(m) == k && all(abs(rowSums(m) - 1) < 1e-6)
}

test_that("wrap_ranger binary classification", {
  skip_if_not_installed("ranger")
  mod <- wrap_ranger(X_bin, y_bin, num.trees = 50L)
  expect_s3_class(mod, "wrap_ranger")
  expect_equal(mod$task, "classification")
  expect_s3_class(predict(mod, newx = X_bin, type = "class"), "factor")
  expect_true(is_prob_matrix(predict(mod, newx = X_bin, type = "prob"), 2L))
})

test_that("wrap_ranger multiclass", {
  skip_if_not_installed("ranger")
  mod <- wrap_ranger(X_multi, y_multi, num.trees = 50L)
  expect_true(is_prob_matrix(predict(mod, newx = X_multi, type = "prob"), 3L))
})

test_that("wrap_ranger regression", {
  skip_if_not_installed("ranger")
  mod <- wrap_ranger(X_reg, y_reg, num.trees = 50L)
  expect_equal(mod$task, "regression")
  expect_true(is.numeric(predict(mod, newx = X_reg)))
})

test_that("wrap_xgboost binary", {
  skip_if_not_installed("xgboost")
  mod <- wrap_xgboost(X_bin, y_bin, nrounds = 20,
                      objective = "binary:logistic", verbose = 0)
  expect_s3_class(predict(mod, newx = X_bin, type = "class"), "factor")
  expect_true(is_prob_matrix(predict(mod, newx = X_bin, type = "prob"), 2L))
})

test_that("wrap_xgboost multiclass", {
  skip_if_not_installed("xgboost")
  mod <- wrap_xgboost(X_multi, y_multi, nrounds = 20,
                      objective = "multi:softprob", num_class = 3, verbose = 0)
  expect_true(is_prob_matrix(predict(mod, newx = X_multi, type = "prob"), 3L))
})

test_that("wrap_xgboost regression", {
  skip_if_not_installed("xgboost")
  mod <- wrap_xgboost(X_reg, y_reg, nrounds = 20,
                      objective = "reg:squarederror", verbose = 0)
  expect_equal(mod$task, "regression")
  expect_true(is.numeric(predict(mod, newx = X_reg)))
})

test_that("wrap_lightgbm binary", {
  skip_if_not_installed("lightgbm")
  mod <- wrap_lightgbm(X_bin, y_bin,
                       params = list(objective = "binary", verbose = -1),
                       nrounds = 20)
  expect_true(is_prob_matrix(predict(mod, newx = X_bin, type = "prob"), 2L))
})

test_that("wrap_lightgbm regression", {
  skip_if_not_installed("lightgbm")
  mod <- wrap_lightgbm(X_reg, y_reg,
                       params = list(objective = "regression", verbose = -1),
                       nrounds = 20)
  expect_equal(mod$task, "regression")
  expect_true(is.numeric(predict(mod, newx = X_reg)))
})

test_that("wrap_glmnet binary", {
  skip_if_not_installed("glmnet")
  mod <- wrap_glmnet(X_bin, y_bin, family = "binomial")
  expect_s3_class(predict(mod, newx = X_bin, type = "class"), "factor")
  expect_true(is_prob_matrix(predict(mod, newx = X_bin, type = "prob"), 2L))
})

test_that("wrap_glmnet regression", {
  skip_if_not_installed("glmnet")
  mod <- wrap_glmnet(X_reg, y_reg, alpha = 0)
  expect_equal(mod$task, "regression")
  expect_true(is.numeric(predict(mod, newx = X_reg)))
})

test_that("wrap_svm binary", {
  skip_if_not_installed("e1071")
  mod <- wrap_svm(X_bin, y_bin, kernel = "radial")
  expect_s3_class(predict(mod, newx = X_bin, type = "class"), "factor")
  expect_true(is_prob_matrix(predict(mod, newx = X_bin, type = "prob"), 2L))
})

test_that("wrap_svm regression", {
  skip_if_not_installed("e1071")
  mod <- wrap_svm(X_reg, y_reg)
  expect_equal(mod$task, "regression")
  expect_true(is.numeric(predict(mod, newx = X_reg)))
})

#' S3 wrapper for e1071 SVM
#'
#' Fits an `e1071` support vector machine with a consistent
#' interface. Supports classification and regression.
#'
#' @param x A matrix or data.frame of features.
#' @param y A factor or character vector for classification, numeric for regression.
#' @param ... Additional arguments passed to [e1071::svm()].
#'   `probability = TRUE` is set automatically for classification; do not
#'   override this if you need `type = "prob"` predictions.
#' @return An object of class `wrap_svm` with fields:
#'   \item{fit}{The fitted svm model.}
#'   \item{levels}{Class levels (NULL for regression).}
#'   \item{task}{"classification" or "regression".}
#' @examples
#'
#' X <- as.matrix(iris[, 1:4])
#' y <- iris$Species
#' mod <- wrap_svm(X, y, kernel = "radial")
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
#' @export
wrap_svm <- function(x, y, ...) {
  if (!requireNamespace("e1071", quietly = TRUE))
    stop("Package 'e1071' required. Install with: install.packages('e1071')")
  x   <- as.matrix(x)
  fit <- e1071::svm(x = x, y = y, probability = .is_classif(y), ...)
  .wrap(fit, y, "wrap_svm")
}

#' @rdname wrap_svm
#' @param object A fitted `wrap_svm` object.
#' @param newx A matrix or data.frame of new observations.
#' @param type `"class"` (default) for class labels, `"prob"` for a probability
#'   matrix. Ignored for regression.
#' @export
#'
#' @examples
#'
#' X <- as.matrix(iris[, 1:4])
#' y <- iris$Species
#' mod <- wrap_svm(X, y, kernel = "radial")
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
predict.wrap_svm <- function(object, newx, type = c("class", "prob"), ...) {
  newx <- as.matrix(newx)
  type <- match.arg(type)
  if (object$task == "regression")
    return(as.numeric(predict(object$fit, newdata = newx)))
  if (type == "prob") {
    raw   <- predict(object$fit, newdata = newx, probability = TRUE)
    probs <- attr(raw, "probabilities")
    if (is.null(probs))
      stop("Model was not trained with probability = TRUE. Refit with wrap_svm().")
    probs <- probs[, object$levels, drop = FALSE]
    return(probs)
  }
  # type == "class": svm returns factor directly
  predict(object$fit, newdata = newx)
}

#' @rdname wrap_svm
#' @export
print.wrap_svm <- function(x, ...) {
  cat("wrap_svm\n")
  cat("  task   :", x$task, "\n")
  if (!is.null(x$levels))
    cat("  classes:", paste(x$levels, collapse = ", "), "\n")
  cat("  kernel :", x$fit$kernel, "\n")
  invisible(x)
}

#' S3 wrapper for glmnet
#'
#' Fits a `glmnet` penalized regression model with a consistent
#' interface. Supports regression and binary classification.
#'
#' @param x A matrix or data.frame of features.
#' @param y A factor or character vector for classification, numeric for regression.
#' @param ... Additional arguments passed to [glmnet::glmnet()].
#'   Pass `family = "binomial"` for binary classification.
#' @return An object of class `wrap_glmnet` with fields:
#'   \item{fit}{The fitted glmnet model.}
#'   \item{levels}{Class levels (NULL for regression).}
#'   \item{task}{"classification" or "regression".}
#' @note Multiclass (`family = "multinomial"`) is not yet supported.
#'   For lambda selection, a specific `s` value can be passed to `predict()`.
#'   By default the midpoint of the lambda path is used. For optimal lambda,
#'   use [glmnet::cv.glmnet()] externally and pass `s = fit$lambda.min`.
#'
#' @examples
#'
#' X <- iris[iris$Species != "virginica", 1:4]
#' y <- droplevels(iris[iris$Species != "virginica", "Species"])
#' mod <- wrap_glmnet(X, y, family = "binomial")
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
#' @export
wrap_glmnet <- function(x, y, ...) {
  if (!requireNamespace("glmnet", quietly = TRUE))
    stop("Package 'glmnet' required. Install with: install.packages('glmnet')")
  x   <- as.matrix(x)
  fit <- glmnet::glmnet(x = x, y = y, ...)
  .wrap(fit, y, "wrap_glmnet")
}

#' @rdname wrap_glmnet
#' @param object A fitted `wrap_glmnet` object.
#' @param newx A matrix or data.frame of new observations.
#' @param type `"class"` (default) for class labels, `"prob"` for a probability
#'   matrix. Ignored for regression.
#' @param s Lambda value for prediction. Defaults to the midpoint of the
#'   lambda path. Pass `s = cv_fit$lambda.min` if using [glmnet::cv.glmnet()].
#' @export
#'
#'
#' @examples
#'
#' X <- iris[iris$Species != "virginica", 1:4]
#' y <- droplevels(iris[iris$Species != "virginica", "Species"])
#' mod <- wrap_glmnet(X, y, family = "binomial")
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
predict.wrap_glmnet <- function(object, newx, type = c("class", "prob"),
                                s = NULL, ...) {
  newx <- as.matrix(newx)
  lvls <- object$levels
  # lambda.min only exists on cv.glmnet — use midpoint of lambda path
  if (is.null(s))
    s <- object$fit$lambda[length(object$fit$lambda) %/% 2L]
  if (object$task == "regression")
    return(drop(predict(object$fit, newx = newx, s = s, type = "response")))
  # Binomial: type="response" gives P(class=2)
  raw   <- drop(predict(object$fit, newx = newx, s = s, type = "response"))
  probs <- cbind(1 - raw, raw)
  colnames(probs) <- lvls
  .format_output(probs, match.arg(type), lvls, object$task)
}

#' @rdname wrap_glmnet
#' @export
print.wrap_glmnet <- function(x, ...) {
  fam <- tryCatch(x$fit$call$family, error = function(e) "gaussian")
  cat("wrap_glmnet\n")
  cat("  task    :", x$task, "\n")
  if (!is.null(x$levels))
    cat("  classes :", paste(x$levels, collapse = ", "), "\n")
  cat("  family  :", deparse(fam), "\n")
  cat("  lambdas :", length(x$fit$lambda), "\n")
  invisible(x)
}

# Coerce newx to either a matrix or a data.frame as required by the backend
#' @keywords internal
.coerce_newx <- function(newx, to = c("matrix", "data.frame")) {
  to <- match.arg(to)
  if (to == "matrix")     return(as.matrix(newx))
  if (to == "data.frame") return(as.data.frame(newx))
}

# Route predict output to the correct format:
#   regression -> pass numeric vector through unchanged
#   type="prob"  -> return named probability matrix as-is
#   type="class" -> convert probability matrix to factor of class labels
#' @keywords internal
.format_output <- function(probs, type, levels, task) {
  if (task == "regression") return(probs)
  type <- match.arg(type, c("class", "prob"))
  if (type == "prob")  return(probs)
  if (type == "class") return(.prob_to_class(probs, levels))
}

# TRUE if y should be treated as a classification target
#' @keywords internal
.is_classif <- function(y) is.factor(y) || is.character(y)

# Convert a probability matrix to a factor of predicted class labels
# by picking the column with the highest probability per row
#' @keywords internal
.prob_to_class <- function(probs, levels) {
  factor(levels[max.col(probs)], levels = levels)
}

# Rename columns to X1, X2, ... so formula-based backends (ranger) always
# see the same names at predict time as at fit time
#' @keywords internal
.std_colnames <- function(x) {
  colnames(x) <- paste0("X", seq_len(ncol(x)))
  x
}

# Coerce character y to factor; leave factors unchanged
#' @keywords internal
.to_factor <- function(y) {
  if (is.character(y)) y <- as.factor(y)
  y
}

# Build the standard wrap_* return object, storing metadata alongside the fit
#' @keywords internal
.wrap <- function(fit, y, class_name, objective = NULL) {
  task   <- if (.is_classif(y)) "classification" else "regression"
  levels <- if (task == "classification") levels(.to_factor(y)) else NULL
  structure(list(fit = fit, levels = levels, task = task, objective = objective),
            class = class_name)
}

# NULL-coalescing operator: return a if non-NULL, otherwise b
#' @keywords internal
`%||%` <- function(a, b) if (!is.null(a)) a else b

#' S3 wrapper for lightgbm
#'
#' Fits a `lightgbm` model with a consistent interface.
#' Supports binary classification, multiclass classification, and regression.
#'
#' @param x A matrix or data.frame of features.
#' @param y A factor or character vector for classification, numeric for regression.
#' @param ... Additional arguments passed to [lightgbm::lgb.train()].
#'   Pass `params = list(objective = "binary")` for binary classification,
#'   `params = list(objective = "multiclass", num_class = k)` for multiclass,
#'   or `params = list(objective = "regression")` for regression.
#' @return An object of class `wrap_lightgbm` with fields:
#'   \item{fit}{The fitted lgb.Booster model.}
#'   \item{levels}{Class levels (NULL for regression).}
#'   \item{task}{"classification" or "regression".}
#'   \item{objective}{The lightgbm objective string, stored at fit time.}
#' @examples
#'
#' library(mlS3)
#'
#' X <- iris[, 1:4]
#' y <- iris$Species
#' mod <- wrap_lightgbm(X, y,
#'   params = list(objective = "multiclass", num_class = 3, verbose = -1),
#'   nrounds = 50)
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
#' @export
wrap_lightgbm <- function(x, y, ...) {
  if (!requireNamespace("lightgbm", quietly = TRUE))
    stop("Package 'lightgbm' required. Install with: install.packages('lightgbm')")
  x     <- as.matrix(x)
  args  <- list(...)
  y_fit <- if (.is_classif(y)) as.numeric(.to_factor(y)) - 1L else as.numeric(y)
  ds    <- lightgbm::lgb.Dataset(data = x, label = y_fit)
  fit   <- lightgbm::lgb.train(data = ds, ...)
  obj   <- args$params$objective %||% args$objective
  .wrap(fit, y, "wrap_lightgbm", objective = obj)
}

#' @rdname wrap_lightgbm
#' @param object A fitted `wrap_lightgbm` object.
#' @param newx A matrix or data.frame of new observations.
#' @param type `"class"` (default) for class labels, `"prob"` for a probability
#'   matrix. Ignored for regression.
#'
#' @export
#'
#' library(mlS3)
#'
#' X <- iris[, 1:4]
#' y <- iris$Species
#' mod <- wrap_lightgbm(X, y,
#'   params = list(objective = "multiclass", num_class = 3, verbose = -1),
#'   nrounds = 50)
#' predict(mod, newx = X, type = "class")
#' predict(mod, newx = X, type = "prob")
#'
predict.wrap_lightgbm <- function(object, newx, type = c("class", "prob"), ...) {
  newx <- as.matrix(newx)
  raw  <- predict(object$fit, newx)
  if (object$task == "regression") return(raw)
  obj  <- object$objective
  if (is.null(obj))
    stop("objective not stored -- refit with wrap_lightgbm()")
  lvls <- object$levels
  k    <- length(lvls)
  if (grepl("binary", obj)) {
    probs <- cbind(1 - raw, raw)
    colnames(probs) <- lvls
    return(.format_output(probs, match.arg(type), lvls, object$task))
  }
  if (grepl("multiclass", obj)) {
    probs <- matrix(raw, nrow = nrow(newx), ncol = k, byrow = TRUE)
    colnames(probs) <- lvls
    return(.format_output(probs, match.arg(type), lvls, object$task))
  }
  raw
}

#' @rdname wrap_lightgbm
#' @export
print.wrap_lightgbm <- function(x, ...) {
  cat("wrap_lightgbm\n")
  cat("  task     :", x$task, "\n")
  if (!is.null(x$levels))
    cat("  classes  :", paste(x$levels, collapse = ", "), "\n")
  cat("  objective:", x$objective, "\n")
  invisible(x)
}

#' Wrap caret models for mlS3
#'
#' Minimal wrapper around caret::train with no tuning.
#' Hyperparameters can be passed via ... as named arguments.
#'
#' @param X Feature matrix or data frame
#' @param y Response vector
#' @param method caret model method (default "rf")
#' @param ... Named hyperparameters (e.g., mtry = 3, ntree = 500)
#'
#' @return Object with class "mlS3_caret"
#' @export
#'
#' @examples
#'
#' # Only runs if caret is installed
#'
#' data(mtcars)
#'
#' # Prepare data
#' X_reg <- mtcars[, -1]  # All except mpg
#' y_reg <- mtcars$mpg     # Target variable
#'
#' # Split into train/test
#' set.seed(123)
#' idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
#' X_train <- X_reg[idx_reg, ]
#' y_train <- y_reg[idx_reg]
#' X_test <- X_reg[-idx_reg, ]
#' y_test <- y_reg[-idx_reg]
#'
#' mod <- wrap_caret(X_train, y_train, method = "rf", mtry = 3)
#' (pred <- predict(mod, X_test))
#'
wrap_caret <- function(X, y, method = "rf", ...) {
  # Check if caret is available - gives helpful error if not
  if (!requireNamespace("caret", quietly = TRUE)) {
    stop(
      "Package 'caret' is required for wrap_caret().\n",
      "Please install it with: install.packages('caret')\n",
      "Alternatively, use other mlS3 wrappers like wrap_xgboost() or wrap_ranger()"
    )
  }

  # Infer task
  task <- if (is.numeric(y)) "regression" else "classification"

  # Ensure factor for classification
  if (task == "classification" && !is.factor(y)) y <- factor(y)

  # Convert ... to tuneGrid if not empty
  dots <- list(...)
  tuneGrid <- if (length(dots) > 0) as.data.frame(dots) else NULL

  # Combine X and y into a data.frame
  df <- data.frame(X, y = y)

  # Train the model
  mod <- caret::train(
    y ~ .,
    data = df,
    method = method,
    trControl = caret::trainControl(method = "none"),
    tuneGrid = tuneGrid
  )

  # Return S3-style object
  structure(
    list(
      model = mod,
      task = task,
      method = method,
      parameters = dots
    ),
    class = "wrap_caret"
  )
}

#' Predict method for mlS3 caret wrapper
#'
#' @param object Object from wrap_caret
#' @param newx New features (matrix or data frame)
#' @param type Prediction type: "raw" (default), "class", "prob", or NULL
#' @param ... Additional arguments to caret::predict.train
#'
#' @return Vector or matrix of predictions
#' @export
#'
#' @examples
#'
#' # Only runs if caret is installed
#'
#' data(mtcars)
#'
#' # Prepare data
#' X_reg <- mtcars[, -1]  # All except mpg
#' y_reg <- mtcars$mpg     # Target variable
#'
#' # Split into train/test
#' set.seed(123)
#' idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
#' X_train <- X_reg[idx_reg, ]
#' y_train <- y_reg[idx_reg]
#' X_test <- X_reg[-idx_reg, ]
#' y_test <- y_reg[-idx_reg]
#'
#' mod <- wrap_caret(X_train, y_train, method = "rf", mtry = 3)
#' (pred <- predict(mod, X_test))
#'
predict.wrap_caret <- function(object, newx, type = NULL, ...) {
  # Check caret availability for prediction too
  if (!requireNamespace("caret", quietly = TRUE)) {
    stop("Package 'caret' is required for predictions from wrap_caret models")
  }

  # Map common mlS3 types to caret types
  caret_type <- if (is.null(type) || type == "class") {
    "raw"
  } else if (type == "prob") {
    if (object$task == "regression") {
      warning("Probability predictions not available for regression tasks, using 'raw'")
      "raw"
    } else {
      "prob"
    }
  } else {
    type
  }

  # Make predictions
  caret::predict.train(object$model, newdata = newx, type = caret_type, ...)
}

#' Print method for wrap_caret objects
#'
#' @param x Object from wrap_caret
#' @param ... Additional arguments
#' @export
print.wrap_caret <- function(x, ...) {
  cat("\n=== mlS3 caret wrapper ===\n")
  cat("Method:", x$method, "\n")
  cat("Task:", x$task, "\n")
  cat("Model class:", class(x$model$finalModel)[1], "\n")

  # Show parameters used
  if (length(x$parameters) > 0) {
    cat("\nParameters:\n")
    param_df <- data.frame(
      Parameter = names(x$parameters),
      Value = unlist(x$parameters),
      row.names = NULL
    )
    print(param_df, row.names = FALSE)
  } else {
    cat("\nParameters: (caret defaults - may involve tuning)\n")
  }

  # Show training info
  cat("\nTraining data shape:", nrow(x$model$trainingData), "rows,",
      ncol(x$model$trainingData) - 1, "features\n")

  # Show call
  cat("\nCall:", deparse(x$call), "\n")

  invisible(x)
}

---
title: "Introduction to R package `mlS3`"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to R package `mlS3`}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

# Classification

```{r fig.width=7}
library(mlS3)

# =============================================================================
# Classification examples (no leakage)
# =============================================================================
set.seed(123)

# --- Binary classification: iris setosa vs versicolor ---
iris_bin <- iris[iris$Species != "virginica", ]
X_bin <- iris_bin[, 1:4]
y_bin <- droplevels(iris_bin$Species)

# Split into train/test
idx_bin <- sample(nrow(X_bin), 0.7 * nrow(X_bin))
X_bin_train <- X_bin[idx_bin, ]
y_bin_train <- y_bin[idx_bin]
X_bin_test  <- X_bin[-idx_bin, ]
y_bin_test  <- y_bin[-idx_bin]

# glmnet
mod <- wrap_glmnet(X_bin_train, y_bin_train, family = "binomial")
pred_bin_glmnet <- predict(mod, newx = X_bin_test, type = "class")
acc_glmnet <- mean(pred_bin_glmnet == y_bin_test)

cat("Accuracy (glmnet): ", acc_glmnet, "\n")


# --- Multiclass classification: iris all species ---
X_multi <- iris[, 1:4]
y_multi <- iris$Species

# Split into train/test
idx_multi <- sample(nrow(X_multi), 0.7 * nrow(X_multi))
X_multi_train <- X_multi[idx_multi, ]
y_multi_train <- y_multi[idx_multi]
X_multi_test  <- X_multi[-idx_multi, ]
y_multi_test  <- y_multi[-idx_multi]

# lightgbm
mod <- wrap_lightgbm(X_multi_train, y_multi_train,
                     params = list(objective = "multiclass",
                                   num_class = 3, verbose = -1),
                     nrounds = 150)
pred_multi_lightgbm <- predict(mod, newx = X_multi_test, type = "class")
acc_lightgbm <- mean(pred_multi_lightgbm == y_multi_test)

# ranger
mod <- wrap_ranger(X_multi_train, y_multi_train, num.trees = 100L)
pred_multi_ranger <- predict(mod, newx = X_multi_test, type = "class")
acc_ranger <- mean(pred_multi_ranger == y_multi_test)

# svm
mod <- wrap_svm(X_multi_train, y_multi_train, kernel = "radial")
pred_multi_svm <- predict(mod, newx = X_multi_test, type = "class")
acc_svm <- mean(pred_multi_svm == y_multi_test)

cat("Accuracy (lightgbm): ", acc_lightgbm, "\n")
cat("Accuracy (ranger): ", acc_ranger, "\n")
cat("Accuracy (svm): ", acc_svm, "\n")
```

# Regression 

```{r fig.width=7}
# =============================================================================
# Regression examples (mtcars)
# =============================================================================
X_reg <- mtcars[, -1]
y_reg <- mtcars$mpg

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_reg_train <- X_reg[idx_reg, ];  y_reg_train <- y_reg[idx_reg]
X_reg_test  <- X_reg[-idx_reg, ]; y_reg_test  <- y_reg[-idx_reg]


# lightgbm
mod <- wrap_lightgbm(X_reg_train, y_reg_train,
                     params = list(objective = "regression", verbose = -1),
                     nrounds = 50)
pred_reg_lightgbm <- predict(mod, newx = X_reg_test)
rmse_lightgbm <- sqrt(mean((pred_reg_lightgbm - y_reg_test)^2))

# glmnet
mod <- wrap_glmnet(X_reg_train, y_reg_train, alpha = 0)
pred_reg_glmnet <- predict(mod, newx = X_reg_test)
rmse_glmnet <- sqrt(mean((pred_reg_glmnet - y_reg_test)^2))

# svm
mod <- wrap_svm(X_reg_train, y_reg_train)
pred_reg_svm <- predict(mod, newx = X_reg_test)
rmse_svm <- sqrt(mean((pred_reg_svm - y_reg_test)^2))

# ranger
mod <- wrap_ranger(X_reg_train, y_reg_train, num.trees = 100L)
pred_reg_ranger <- predict(mod, newx = X_reg_test)
rmse_ranger <- sqrt(mean((pred_reg_ranger - y_reg_test)^2))

cat("RMSE (lightgbm): ", rmse_lightgbm, "\n")
cat("RMSE (glmnet): ", rmse_glmnet, "\n")
cat("RMSE (svm): ", rmse_svm, "\n")
cat("RMSE (ranger): ", rmse_ranger, "\n")
```

---
title: "Introduction to R package `mlS3` -- with caret"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Introduction to R package `mlS3`  -- with caret}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---

Model parameters available for `caret`: [https://topepo.github.io/caret/available-models.html](https://topepo.github.io/caret/available-models.html)

# Regression

```{r regression, fig.width=7}
# Load required libraries
library(mlS3)
library(caret)  # Will be suggested, but needed for this example

# ============================================================================
# Regression with mtcars dataset
# ============================================================================
data(mtcars)

# Prepare data
X_reg <- mtcars[, -1]  # All except mpg
y_reg <- mtcars$mpg     # Target variable

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_reg_train <- X_reg[idx_reg, ]
y_reg_train <- y_reg[idx_reg]
X_reg_test <- X_reg[-idx_reg, ]
y_reg_test <- y_reg[-idx_reg]

# ----------------------------------------------------------------------------
# Example 1: Random Forest with specific parameters
# ----------------------------------------------------------------------------
cat("\n=== Example 1: Random Forest Regression ===\n")

mod_rf <- wrap_caret(X_reg_train, y_reg_train, 
                     method = "rf", 
                     mtry = 3)        # Number of variables sampled at each split

print(mod_rf)

# Predictions
pred_rf <- predict(mod_rf, newx = X_reg_test)
rmse_rf <- sqrt(mean((pred_rf - y_reg_test)^2))
r2_rf <- 1 - sum((y_reg_test - pred_rf)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_rf, 3), "\n")
cat("R-squared:", round(r2_rf, 3), "\n")

# ----------------------------------------------------------------------------
# Example 3: Support Vector Machine
# ----------------------------------------------------------------------------
cat("\n=== Example 3: SVM Regression ===\n")

mod_svm <- wrap_caret(X_reg_train, y_reg_train,
                      method = "svmRadial",
                      sigma = 0.1,
                      C = 1)

print(mod_svm)

pred_svm <- predict(mod_svm, newx = X_reg_test)
rmse_svm <- sqrt(mean((pred_svm - y_reg_test)^2))
r2_svm <- 1 - sum((y_reg_test - pred_svm)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_svm, 3), "\n")
cat("R-squared:", round(r2_svm, 3), "\n")

# ----------------------------------------------------------------------------
# Example 4: Neural Network
# ----------------------------------------------------------------------------
cat("\n=== Example 4: Neural Network Regression ===\n")

mod_nnet <- wrap_caret(X_reg_train, y_reg_train,
                       method = "nnet",
                       size = 5,      # Number of hidden units
                       decay = 0.01)  # Weight decay

print(mod_nnet)

pred_nnet <- predict(mod_nnet, newx = X_reg_test)
rmse_nnet <- sqrt(mean((pred_nnet - y_reg_test)^2))
r2_nnet <- 1 - sum((y_reg_test - pred_nnet)^2) / sum((y_reg_test - mean(y_reg_test))^2)

cat("RMSE:", round(rmse_nnet, 3), "\n")
cat("R-squared:", round(r2_nnet, 3), "\n")

# ----------------------------------------------------------------------------
# Example 5: Compare all models
# ----------------------------------------------------------------------------
cat("\n=== Model Comparison ===\n")
results_reg <- data.frame(
  Model = c("Random Forest", "SVM", "Neural Network"),
  RMSE = round(c(rmse_rf, rmse_svm, rmse_nnet), 3),
  R2 = round(c(r2_rf, r2_svm, r2_nnet), 3)
)
print(results_reg)

# Best model
best_idx <- which.min(results_reg$RMSE)
cat("\nBest model (lowest RMSE):", results_reg$Model[best_idx], "\n")
```

# Classification

```{r classification, fig.width=7}
# ============================================================================
# Classification with iris dataset
# ============================================================================
data(iris)

# ----------------------------------------------------------------------------
# Example 1: Binary Classification (setosa vs versicolor)
# ----------------------------------------------------------------------------
cat("\n=== Example 1: Binary Classification ===\n")

iris_bin <- iris[iris$Species != "virginica", ]
X_bin <- iris_bin[, 1:4]
y_bin <- droplevels(iris_bin$Species)

# Split
set.seed(123)
idx_bin <- sample(nrow(X_bin), 0.7 * nrow(X_bin))
X_bin_train <- X_bin[idx_bin, ]
y_bin_train <- y_bin[idx_bin]
X_bin_test <- X_bin[-idx_bin, ]
y_bin_test <- y_bin[-idx_bin]

# Random Forest
mod_rf_bin <- wrap_caret(X_bin_train, y_bin_train,
                         method = "rf",
                         mtry = 2)

print(mod_rf_bin)

# Class predictions
pred_rf_bin <- predict(mod_rf_bin, newx = X_bin_test, type = "class")
acc_rf <- mean(pred_rf_bin == y_bin_test)

# Probability predictions
prob_rf_bin <- predict(mod_rf_bin, newx = X_bin_test, type = "prob")
head(prob_rf_bin)

cat("Accuracy:", round(acc_rf, 4), "\n")
cat("Confusion Matrix:\n")
print(table(Predicted = pred_rf_bin, Actual = y_bin_test))

# ----------------------------------------------------------------------------
# Example 2: Multiclass Classification (all species)
# ----------------------------------------------------------------------------
cat("\n=== Example 2: Multiclass Classification ===\n")

X_multi <- iris[, 1:4]
y_multi <- iris$Species

# Split
set.seed(123)
idx_multi <- sample(nrow(X_multi), 0.7 * nrow(X_multi))
X_multi_train <- X_multi[idx_multi, ]
y_multi_train <- y_multi[idx_multi]
X_multi_test <- X_multi[-idx_multi, ]
y_multi_test <- y_multi[-idx_multi]

# Random Forest
mod_rf_multi <- wrap_caret(X_multi_train, y_multi_train,
                           method = "rf",
                           mtry = 2)

pred_rf_multi <- predict(mod_rf_multi, newx = X_multi_test, type = "class")
acc_rf_multi <- mean(pred_rf_multi == y_multi_test)
cat("Random Forest Accuracy:", round(acc_rf_multi, 4), "\n")

# ----------------------------------------------------------------------------
# Example 3: Support Vector Machine (multiclass)
# ----------------------------------------------------------------------------
cat("\n=== Example 3: SVM Multiclass ===\n")

mod_svm_multi <- wrap_caret(X_multi_train, y_multi_train,
                            method = "svmRadial",
                            sigma = 0.1,
                            C = 10)

pred_svm_multi <- predict(mod_svm_multi, newx = X_multi_test, type = "class")
acc_svm_multi <- mean(pred_svm_multi == y_multi_test)
cat("SVM Accuracy:", round(acc_svm_multi, 4), "\n")

# Confusion Matrix
cat("\nSVM Confusion Matrix:\n")
print(table(Predicted = pred_svm_multi, Actual = y_multi_test))

# ----------------------------------------------------------------------------
# Example 4: K-Nearest Neighbors
# ----------------------------------------------------------------------------
cat("\n=== Example 4: KNN Classification ===\n")

mod_knn <- wrap_caret(X_multi_train, y_multi_train,
                      method = "knn",
                      k = 5)

pred_knn <- predict(mod_knn, newx = X_multi_test, type = "class")
acc_knn <- mean(pred_knn == y_multi_test)
cat("KNN Accuracy:", round(acc_knn, 4), "\n")

# ----------------------------------------------------------------------------
# Example 5: Linear Discriminant Analysis
# ----------------------------------------------------------------------------
cat("\n=== Example 5: LDA Classification ===\n")

mod_lda <- wrap_caret(X_multi_train, y_multi_train,
                      method = "lda")

pred_lda <- predict(mod_lda, newx = X_multi_test, type = "class")
acc_lda <- mean(pred_lda == y_multi_test)
cat("LDA Accuracy:", round(acc_lda, 4), "\n")

# Get posterior probabilities
prob_lda <- predict(mod_lda, newx = X_multi_test, type = "prob")
cat("\nSample probabilities (first 3 rows):\n")
print(head(prob_lda, 3))

# ----------------------------------------------------------------------------
# Example 7: Compare all classification models
# ----------------------------------------------------------------------------
cat("\n=== Model Comparison (Multiclass) ===\n")
results_class <- data.frame(
  Model = c("Random Forest", "SVM", "KNN", "LDA"),
  Accuracy = round(c(acc_rf_multi, acc_svm_multi, acc_knn, acc_lda), 4)
)
print(results_class[order(-results_class$Accuracy), ])

# ----------------------------------------------------------------------------
# Example 8: Advanced - Probability calibration check
# ----------------------------------------------------------------------------
cat("\n=== Probability Distributions (LDA) ===\n")

# Get probabilities for all classes
probs <- predict(mod_lda, newx = X_multi_test, type = "prob")

# Show distribution of predicted probabilities for each class
oldpar <- par(mfrow = c(1, 3))
on.exit(par(oldpar))
for (class_name in colnames(probs)) {
  hist(probs[, class_name], 
       main = paste("Probabilities for", class_name),
       xlab = "Probability",
       col = "lightblue",
       breaks = 20)
}
```

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_svm.R
\name{wrap_svm}
\alias{wrap_svm}
\alias{predict.wrap_svm}
\alias{print.wrap_svm}
\title{S3 wrapper for e1071 SVM}
\usage{
wrap_svm(x, y, ...)

\method{predict}{wrap_svm}(object, newx, type = c("class", "prob"), ...)

\method{print}{wrap_svm}(x, ...)
}
\arguments{
\item{x}{A matrix or data.frame of features.}

\item{y}{A factor or character vector for classification, numeric for regression.}

\item{...}{Additional arguments passed to [e1071::svm()].
`probability = TRUE` is set automatically for classification; do not
override this if you need `type = "prob"` predictions.}

\item{object}{A fitted `wrap_svm` object.}

\item{newx}{A matrix or data.frame of new observations.}

\item{type}{`"class"` (default) for class labels, `"prob"` for a probability
matrix. Ignored for regression.}
}
\value{
An object of class `wrap_svm` with fields:
  \item{fit}{The fitted svm model.}
  \item{levels}{Class levels (NULL for regression).}
  \item{task}{"classification" or "regression".}
}
\description{
Fits an `e1071` support vector machine with a consistent
interface. Supports classification and regression.
}
\examples{

X <- as.matrix(iris[, 1:4])
y <- iris$Species
mod <- wrap_svm(X, y, kernel = "radial")
predict(mod, newx = X, type = "class")
predict(mod, newx = X, type = "prob")


X <- as.matrix(iris[, 1:4])
y <- iris$Species
mod <- wrap_svm(X, y, kernel = "radial")
predict(mod, newx = X, type = "class")
predict(mod, newx = X, type = "prob")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_lightgbm.R
\name{wrap_lightgbm}
\alias{wrap_lightgbm}
\alias{predict.wrap_lightgbm}
\alias{print.wrap_lightgbm}
\title{S3 wrapper for lightgbm}
\usage{
wrap_lightgbm(x, y, ...)

\method{predict}{wrap_lightgbm}(object, newx, type = c("class", "prob"), ...)

\method{print}{wrap_lightgbm}(x, ...)
}
\arguments{
\item{x}{A matrix or data.frame of features.}

\item{y}{A factor or character vector for classification, numeric for regression.}

\item{...}{Additional arguments passed to [lightgbm::lgb.train()].
Pass `params = list(objective = "binary")` for binary classification,
`params = list(objective = "multiclass", num_class = k)` for multiclass,
or `params = list(objective = "regression")` for regression.}

\item{object}{A fitted `wrap_lightgbm` object.}

\item{newx}{A matrix or data.frame of new observations.}

\item{type}{`"class"` (default) for class labels, `"prob"` for a probability
matrix. Ignored for regression.}
}
\value{
An object of class `wrap_lightgbm` with fields:
  \item{fit}{The fitted lgb.Booster model.}
  \item{levels}{Class levels (NULL for regression).}
  \item{task}{"classification" or "regression".}
  \item{objective}{The lightgbm objective string, stored at fit time.}
}
\description{
Fits a `lightgbm` model with a consistent interface.
Supports binary classification, multiclass classification, and regression.
}
\examples{

library(mlS3)

X <- iris[, 1:4]
y <- iris$Species
mod <- wrap_lightgbm(X, y,
  params = list(objective = "multiclass", num_class = 3, verbose = -1),
  nrounds = 50)
predict(mod, newx = X, type = "class")
predict(mod, newx = X, type = "prob")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_glmnet.R
\name{wrap_glmnet}
\alias{wrap_glmnet}
\alias{predict.wrap_glmnet}
\alias{print.wrap_glmnet}
\title{S3 wrapper for glmnet}
\usage{
wrap_glmnet(x, y, ...)

\method{predict}{wrap_glmnet}(object, newx, type = c("class", "prob"), s = NULL, ...)

\method{print}{wrap_glmnet}(x, ...)
}
\arguments{
\item{x}{A matrix or data.frame of features.}

\item{y}{A factor or character vector for classification, numeric for regression.}

\item{...}{Additional arguments passed to [glmnet::glmnet()].
Pass `family = "binomial"` for binary classification.}

\item{object}{A fitted `wrap_glmnet` object.}

\item{newx}{A matrix or data.frame of new observations.}

\item{type}{`"class"` (default) for class labels, `"prob"` for a probability
matrix. Ignored for regression.}

\item{s}{Lambda value for prediction. Defaults to the midpoint of the
lambda path. Pass `s = cv_fit$lambda.min` if using [glmnet::cv.glmnet()].}
}
\value{
An object of class `wrap_glmnet` with fields:
  \item{fit}{The fitted glmnet model.}
  \item{levels}{Class levels (NULL for regression).}
  \item{task}{"classification" or "regression".}
}
\description{
Fits a `glmnet` penalized regression model with a consistent
interface. Supports regression and binary classification.
}
\note{
Multiclass (`family = "multinomial"`) is not yet supported.
  For lambda selection, a specific `s` value can be passed to `predict()`.
  By default the midpoint of the lambda path is used. For optimal lambda,
  use [glmnet::cv.glmnet()] externally and pass `s = fit$lambda.min`.
}
\examples{

X <- iris[iris$Species != "virginica", 1:4]
y <- droplevels(iris[iris$Species != "virginica", "Species"])
mod <- wrap_glmnet(X, y, family = "binomial")
predict(mod, newx = X, type = "class")
predict(mod, newx = X, type = "prob")


X <- iris[iris$Species != "virginica", 1:4]
y <- droplevels(iris[iris$Species != "virginica", "Species"])
mod <- wrap_glmnet(X, y, family = "binomial")
predict(mod, newx = X, type = "class")
predict(mod, newx = X, type = "prob")

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_caret.R
\name{predict.wrap_caret}
\alias{predict.wrap_caret}
\title{Predict method for mlS3 caret wrapper}
\usage{
\method{predict}{wrap_caret}(object, newx, type = NULL, ...)
}
\arguments{
\item{object}{Object from wrap_caret}

\item{newx}{New features (matrix or data frame)}

\item{type}{Prediction type: "raw" (default), "class", "prob", or NULL}

\item{...}{Additional arguments to caret::predict.train}
}
\value{
Vector or matrix of predictions
}
\description{
Predict method for mlS3 caret wrapper
}
\examples{

# Only runs if caret is installed

data(mtcars)

# Prepare data
X_reg <- mtcars[, -1]  # All except mpg
y_reg <- mtcars$mpg     # Target variable

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_train <- X_reg[idx_reg, ]
y_train <- y_reg[idx_reg]
X_test <- X_reg[-idx_reg, ]
y_test <- y_reg[-idx_reg]

mod <- wrap_caret(X_train, y_train, method = "rf", mtry = 3)
(pred <- predict(mod, X_test))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_caret.R
\name{print.wrap_caret}
\alias{print.wrap_caret}
\title{Print method for wrap_caret objects}
\usage{
\method{print}{wrap_caret}(x, ...)
}
\arguments{
\item{x}{Object from wrap_caret}

\item{...}{Additional arguments}
}
\description{
Print method for wrap_caret objects
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wrap_caret.R
\name{wrap_caret}
\alias{wrap_caret}
\title{Wrap caret models for mlS3}
\usage{
wrap_caret(X, y, method = "rf", ...)
}
\arguments{
\item{X}{Feature matrix or data frame}

\item{y}{Response vector}

\item{method}{caret model method (default "rf")}

\item{...}{Named hyperparameters (e.g., mtry = 3, ntree = 500)}
}
\value{
Object with class "mlS3_caret"
}
\description{
Minimal wrapper around caret::train with no tuning.
Hyperparameters can be passed via ... as named arguments.
}
\examples{

# Only runs if caret is installed

data(mtcars)

# Prepare data
X_reg <- mtcars[, -1]  # All except mpg
y_reg <- mtcars$mpg     # Target variable

# Split into train/test
set.seed(123)
idx_reg <- sample(nrow(X_reg), 0.7 * nrow(X_reg))
X_train <- X_reg[idx_reg, ]
y_train <- y_reg[idx_reg]
X_test <- X_reg[-idx_reg, ]
y_test <- y_reg[-idx_reg]

mod <- wrap_caret(X_train, y_train, method = "rf", mtry = 3)
(pred <- predict(mod, X_test))

}
